@@ -76,6 +76,8 @@ typedef enum {
 #endif
     MIB_IP6,
     MIB_IP6_ADDR,
+    MIB_IP6_ROUTE,
+    MIB_ICMP6,
     MIB_TCP6_CONN,
     MIB_UDP6_ENDPOINT,
 #endif
